package groowt.view.component.web.compiler;

import groowt.view.component.ViewComponent;
import groowt.view.component.compiler.AbstractComponentTemplateCompileUnit;
import groowt.view.component.compiler.ComponentTemplateCompileException;
import groowt.view.component.compiler.ComponentTemplateCompileResult;
import groowt.view.component.compiler.source.ComponentTemplateSource;
import groowt.view.component.compiler.source.FileSource;
import groowt.view.component.compiler.source.URISource;
import groowt.view.component.compiler.source.URLSource;
import org.codehaus.groovy.control.CompilationUnit;
import org.codehaus.groovy.control.CompilerConfiguration;
import org.codehaus.groovy.control.Janitor;
import org.codehaus.groovy.control.customizers.ImportCustomizer;
import org.codehaus.groovy.control.io.ReaderSource;
import org.jetbrains.annotations.Nullable;

import java.io.Reader;
import java.net.URI;

public class DefaultWebViewComponentTemplateCompileUnit extends AbstractComponentTemplateCompileUnit
        implements ReaderSource, WebViewComponentTemplateCompileUnit {

    private static CompilationUnit getCompilationUnit() {
        final var configuration = new CompilerConfiguration();
        final var addGroowtImports = new ImportCustomizer();
        addGroowtImports.addStarImports(
                "groowt.view.component.web.lib",
                "groowt.view.component.web.runtime",
                "groowt.view.component.runtime"
        );
        configuration.addCompilationCustomizers(addGroowtImports);
        return new CompilationUnit(configuration);
    }

    private final String defaultPackageName;
    private final CompilationUnit groovyCompilationUnit;

    public DefaultWebViewComponentTemplateCompileUnit(
            String descriptiveName,
            Class<? extends ViewComponent> forClass,
            ComponentTemplateSource source,
            String defaultPackageName
    ) {
        super(descriptiveName, forClass, source);
        if (!defaultPackageName.isEmpty() && !defaultPackageName.endsWith(".")) {
            this.defaultPackageName = defaultPackageName + ".";
        } else {
            this.defaultPackageName = defaultPackageName;
        }
        this.groovyCompilationUnit = getCompilationUnit();
    }

    @Override
    public String getDefaultPackageName() {
        return this.defaultPackageName;
    }

    @Override
    public CompilationUnit getGroovyCompilationUnit() {
        return this.groovyCompilationUnit;
    }

    @Override
    public ReaderSource getGroovyReaderSource() {
        return this;
    }

    @Override
    public ComponentTemplateCompileResult compile() throws ComponentTemplateCompileException {
        final WebViewComponentTemplateCompiler compiler = WebViewComponentTemplateCompiler.get();
        return compiler.compile(this);
    }

    @Override
    public Reader getReader() {
        try {
            return this.getSource().toReader();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public boolean canReopenSource() {
        return this.getSource().canReopen();
    }

    @Override
    public @Nullable String getLine(int lineNumber, Janitor janitor) {
        if (lineNumber > -1 && this.getSource().canReopen()) {
            return this.getSource().getLines().get(lineNumber);
        } else {
            return null;
        }
    }

    @Override
    public void cleanup() {}

    @Override
    public @Nullable URI getURI() {
        return switch (this.getSource()) {
            case FileSource fileSource -> fileSource.getURI();
            case URISource uriSource -> uriSource.getURI();
            case URLSource urlSource -> urlSource.getURI();
            default -> null;
        };
    }

}
